package cn.com.qmth.print.manage.service.impl;

import cn.com.qmth.print.manage.dao.UserDao;
import cn.com.qmth.print.manage.entity.UserEntity;
import cn.com.qmth.print.manage.enums.RoleMeta;
import cn.com.qmth.print.manage.service.UserService;
import cn.com.qmth.print.manage.service.query.UserQuery;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Date: 2021/11/16.
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserDao, UserEntity> implements UserService {

    @Override
    public IPage<UserEntity> pageQuery(UserQuery query) {
        QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(query.getLoginName())) {
            queryWrapper.lambda().like(UserEntity::getLoginName, query.getLoginName());
        }
        if (query.getRole() != null) {
            queryWrapper.lambda().eq(UserEntity::getRole, query.getRole());
        }
        queryWrapper.lambda().eq(UserEntity::isEnable, query.isEnable()).orderByAsc(UserEntity::getId);
        Page<UserEntity> page = new Page<>(query.getPageNumber(), query.getPageSize());
        return this.page(page, queryWrapper);
    }

    @Override
    public List<UserEntity> listByEnable() {
        QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(UserEntity::getRole, RoleMeta.CHECKER).eq(UserEntity::isEnable, true);
        return this.list(queryWrapper);
    }

    @Override
    public UserEntity findByLoginName(String loginName) {
        QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(UserEntity::getLoginName, loginName);
        return this.getOne(queryWrapper);
    }
}
